package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.OpcionCombate
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.Efecto
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoAtaqueEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoAtaqueProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoCriticoAutomaticoEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoHabilidadesProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoHuirProtagonista
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoInvestigarEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoRegeneracionEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoRevivirEnemigo
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.Objeto
import com.example.ellegadodepintia.repositorios.RepositorioObjetos

data class CombateInfo(
    val imagenIdle: Int,
    val imagenAtaque: Int,
    val habilidades: List<Efecto>,
    val vidaMax: Int,
    val ataque: Int,
    val recompensa: Objeto
)

class CombateFactory : EventoFactory {

    private val combatesMap = mapOf(
        "El Metal Volador es una criatura formada por fragmentos de metal antiguo. Protege los secretos del yacimiento de Pintia." to CombateInfo(
            R.drawable.monster_metal_idle_animation,
            R.drawable.monster_metal_attack_animation,
            listOf(
                EfectoAtaqueEnemigo(80),
                EfectoRegeneracionEnemigo(
                    "El metal reconfigura sus fragmentos recuperando parte de su vida",
                    (1..2).random()
                )
            ),
            vidaMax = (5..10).random(),
            ataque = (1..2).random(),
            RepositorioObjetos.obtenerObjetoPorNombre("Moneda de plata Vaccea")!!
        ),
        "Al explorar una tumba de la necrópolis de las ruedas, un esqueleto armado aparece." to CombateInfo(
            imagenIdle = R.drawable.monster_skeleton_idle,
            imagenAtaque = R.drawable.monster_skeleton_attack,
            habilidades = listOf(EfectoAtaqueEnemigo(75), EfectoRevivirEnemigo("El esqueleto se reconstruye reviviendo",0.35, 40)),
            vidaMax = (5..8).random(),
            ataque = (1..3).random(),
            RepositorioObjetos.obtenerObjetoPorNombre("Poción de Veneno II")!!
        ),
        "Te encuentras con el Lobo Vacceo, sus ojos amarillos te observan fijamente, llenos de una sabiduría ancestral y una amenaza latente" to CombateInfo(
            imagenIdle = R.drawable.monster_wolf_idle_animation,
            imagenAtaque = R.drawable.monster_wolf_attack_animation,
            habilidades = listOf(EfectoAtaqueEnemigo(80), EfectoCriticoAutomaticoEnemigo("El lobo te golpea gravemente con sus enormes zarpas")),
            vidaMax = (7..11).random(),
            ataque = (2..3).random(),
            RepositorioObjetos.obtenerObjetoPorNombre("Piel del Gran Lobo")!!
        )
    )

    override fun generarDescripcion(): String {
        return combatesMap.keys.random()
    }

    override fun generarEvento(): EventoCombate {
        val descripcion = generarDescripcion()
        val combateInfo = combatesMap[descripcion]
            ?: throw IllegalArgumentException("No se encontró información de combate")

        return EventoCombate(
            descripcion = descripcion,
            dificultad = 0,
            opciones = generarOpciones(""),
            imagenAtaque = combateInfo.imagenAtaque,
            imagen = combateInfo.imagenIdle,
            resistenciaMax = combateInfo.vidaMax,
            ataque = combateInfo.ataque,
            finalizado = false,
            acciones = combateInfo.habilidades.toMutableList(),
            recompensa = combateInfo.recompensa
        )
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        return mutableListOf(
            OpcionCombate(
                descripcion = "Atacar",
                atributoRequerida = Atributo.Ataque,
                efecto = EfectoAtaqueProtagonista(70)
            ),
            Opcion(
                descripcion = "Investigar",
                atributoRequerida = Atributo.Investigacion,
                efecto = EfectoInvestigarEnemigo(100)
            ),
            Opcion(
                descripcion = "Habilidades",
                atributoRequerida = Atributo.Nula,
                efecto = EfectoHabilidadesProtagonista(GameState.jugador.atributos[Atributo.Vida]!!)
            ),
            OpcionCombate(
                descripcion = "Huir",
                atributoRequerida = Atributo.Vida,
                efecto = EfectoHuirProtagonista(GameState.jugador.atributos[Atributo.Vida]!!),
            )
        )
    }

    override fun generarImagen(descripcion: String): Int {
        return combatesMap[descripcion]!!.imagenIdle
    }

}
