    package com.example.ellegadodepintia.repositorios

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.Habilidad
import com.example.ellegadodepintia.exploradoresDePintia.model.habilidad.efectoHabilidad.EfectoBuffo
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.*
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoEquipableAumento
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoEquipableHabilidad
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoObjetoAtacar
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.efectoObjeto.EfectoObjetoCura

object RepositorioObjetos {
    private val objetos: Map<String, Objeto> by lazy {
        listOf(
            //******************************************
            // E    Q   U   I   P   A   B   L   E   S
            //******************************************
            ObjetoEquipable(
                nombre = "Espada de Hierro",
                descripcion = "Una espada de hierro sencilla y confiable perfecta para un aventurero novato. Otorga 1 de daño extra",
                efecto = EfectoEquipableAumento(1, Atributo.Ataque),
                imagen = R.drawable.asi_objeto_espadadefault
            ),
            ObjetoEquipable(
                nombre = "Casco Ceremonial",
                descripcion = "Casco ceremonial de oro de los vacceos decorado con intrincados patrones que simboliza el estatus social. Aumenta tu vida máxima en 3 puntos",
                efecto = EfectoEquipableAumento(3, Atributo.Vida),
                imagen = R.drawable.asi_objeto_casco
            ),
            ObjetoEquipable(
                nombre = "Escudo Caetrae",
                descripcion = "La caetrae es un escudo redondo vacceo, forjado para la defensa en combate, que aumenta tu vida máxima en 5 puntos.",
                efecto = EfectoEquipableAumento(5, Atributo.Vida),
                imagen = R.drawable.asi_objeto_escudo
            ),
            ObjetoEquipable(
                nombre = "Puñal Vacceo",
                descripcion = "Espada corta, típica de diseño celtibérico. Mientras está equipada añade la habilidad \"Letalidad\" para su uso en combate.",
                efecto = EfectoEquipableHabilidad(
                    Habilidad(
                        "Letalidad",
                        "Aumenta la probabilidad de realizar un golpe crítico a un 30% por dos turnos",
                        EfectoBuffo(Atributo.Critico, 22, 2),
                        energia = 2
                    )
                ),
                imagen = R.drawable.asi_objeto_daga
            ),
            //**********************************************
            //  C   O   N   S   U   M   I   B   L   E   S
            //**********************************************
            ObjetoConsumible(
                nombre = "Elixir Menor de la Necrópolis",
                descripcion = "Un frasco pequeño con un líquido granate y brillante, que destella con un resplandor profundo, capaz de restaurar 3 puntos de vida al instante.",
                efecto = EfectoObjetoCura(3),
                imagen = R.drawable.asi_objeto_pocima_vida_5
            ),
            ObjetoConsumible(
                nombre = "Elixir Mediano de la Necrópolis",
                descripcion = "Un frasco robusto con un líquido amarillo brillante, que resplandece como el sol al mediodía, capaz de restaurar 5 puntos de vida al instante.",
                efecto = EfectoObjetoCura(5),
                imagen = R.drawable.asi_objeto_pocima_vida_10
            ),
            ObjetoConsumible(
                nombre = "Elixir Supremo de la Necrópolis",
                descripcion = "Frasco supremo forjado con los secretos de los vacceos, restaura toda tu vitalidad al instante, ¡la poción de cura más poderosa de Pintia!",
                efecto = EfectoObjetoCura(999),
                imagen = R.drawable.asi_objeto_pocima_vida_all
            ),
            ObjetoConsumible(
                nombre = "Poción de Veneno II",
                descripcion = "Según las leyendas, los vacceos preparaban este brebaje para impregnar sus armas, asegurando que incluso un rasguño pudiera derrotar a un enemigo. Hace 2 de daño al enemigo.",
                efecto = EfectoObjetoAtacar(2),
                imagen = R.drawable.asi_objeto_veneno2
            ),
            //**********************************
            //  D   E       V   A   L   O   R
            //**********************************
            ObjetoDeValor(
                nombre = "Vasito de barro cocido",
                descripcion = "Objeto de gran valor. Se convierte en 10 monedas si consigues completar la partida.",
                coste = 10,
                imagen = R.drawable.asi_exploradores_vasito
            ),
            ObjetoDeValor(
                nombre = "Jabonera de cerámica",
                descripcion = "Objeto de gran valor. Se convierte en 15 monedas si consigues completar la partida.",
                coste = 15,
                imagen = R.drawable.asi_exploradores_jabonera
            ),
            ObjetoDeValor(
                nombre = "Copa negra con inscripciones",
                descripcion = "Objeto de gran valor. Se convierte en 15 monedas si consigues completar la partida.",
                coste = 15,
                imagen = R.drawable.asi_minijuego_3_copa_negra_brunida
            ),
            ObjetoDeValor(
                nombre = "Tapadera de cerámica zoomorfa",
                descripcion = "Objeto de gran valor. Se convierte en 30 monedas si consigues completar la partida.",
                coste = 30,
                imagen = R.drawable.asi_minijuego_3_tapadera_zoomorfo
            ),
            ObjetoDeValor(
                nombre= "Moneda de plata Vaccea",
                descripcion = "Pequeño denario con símbolos celtíberos, utilizadas para el comercio. Su circulación refleja una economía organizada en torno a la élite guerrera",
                imagen = R.drawable.asi_objeto_moneda,
                coste = 15
            ),
            ObjetoDeValor(
                nombre= "Sonajero Redondo",
                descripcion = "Pequeño artefacto de cerámica utilizado en rituales funerarios o ceremonias. Su diseño y sonido característico eran símbolos de protección y conexión espiritual en la cultura celtíbera.",
                imagen = R.drawable.asi_minijuego_3_sonajero_redondo,
                coste = 15
            ),
            ObjetoDeValor(
                nombre= "Piel del Gran Lobo",
                descripcion = "Piel obtenida del lobo que representa parte de la mitología vaccea. Está valorado en 100 monedas",
                imagen = R.drawable.asi_minijuego_3_sonajero_redondo,
                coste = 100
            ),
            //**********************************************
            //  D   E       U   T   I   L   I   D   A   D
            //**********************************************
            ObjetoUtilidad(
                nombre = "Pico Vacceo",
                descripcion = "Un pico viejo y oscuro con dibujos extraños que parecen brillar cerca de las piedras, como si estuviera esperando que lo uses para algo importante.",
                imagen = R.drawable.asi_objeto_pico
            ),
            ObjetoUtilidad(
                nombre = "Ticket",
                descripcion = "Representa una invitación a un evento relacionado con la cultura vaccea, como una feria arqueológica o un festival cultural",
                imagen = R.drawable.asi_objeto_ticket
            ),
            ObjetoUtilidad(
                nombre = "Pala Vaccea",
                descripcion = "Una antigua pala con marcas de uso, forjada con madera resistente y un metal desgastado por el tiempo. Parece estar llena de historia.",
                imagen = R.drawable.asi_objeto_pala
            )
        ).associateBy { it.nombre }
    }

    fun obtenerObjetoPorNombre(nombre: String): Objeto? {
        return objetos[nombre]
    }

    fun obtenerObjetoPorCoste(coste: Int): Objeto? {
        val objetosFiltrados = objetos.values
            .filterIsInstance<ObjetoDeValor>()
            .filter { it.coste <= coste }
            .groupBy { it.coste }

        if (objetosFiltrados.isEmpty()){
            return null
        }

        val costeMasCercano = objetosFiltrados.keys
            .filter { it <= coste }
            .maxOrNull()

        val objetosConCosteMasCercano = objetosFiltrados[costeMasCercano] ?: return null
        return objetosConCosteMasCercano.random()
    }




    fun obtenerObjetosAleatoriosTienda(cantidad: Int): ArrayList<String> {
        val consumibles = objetos.values.filter { it is ObjetoConsumible || it is ObjetoUtilidad }
        return ArrayList<String>().apply {
            repeat(cantidad) {
                add((consumibles.random()).nombre)
            }
        }
    }

    fun obtenerObjetosAleatoriosArmeria(cantidad: Int): ArrayList<String> {
        val equipables = objetos.values.filterIsInstance<ObjetoEquipable>()
        return ArrayList<String>().apply {
            repeat(cantidad) {
                println((equipables.random()).nombre)
                add((equipables.random()).nombre)
            }
        }
    }

    fun convertirNombresAObjetos(nombres: List<String>): List<Objeto?> {
        return nombres.map { nombre ->
            obtenerObjetoPorNombre(nombre)
        }
    }
}